/* *********************************************************
 *  RGB Colour Display Example
 *  
 *  Controls the colour of a square panel and the values of
 *  Red, Green, and Blue number displays on a MegunoLink 
 *  interface panel according to RGB colour sensor 
 *  measurements. Load the companion project: 
 *  "RGBSensorDisplay.mlx" into MegunoLink.
*  ****************************************************** */


//--------------------Includes-------------------------
#include <Wire.h>

// Sensor library, install if missing:
//"Sketch"->"Include Library"->"Manage Libraries"-> search
#include "Adafruit_TCS34725.h" // Adafruit TCS34725

// MegunoLink library, install if missing as above.
#include "MegunoLink.h" // Megunolink


//--------------------Globals-------------------------
// Create class instance for reading this colour sensor.
Adafruit_TCS34725 ColourSensor = Adafruit_TCS34725();
// SCL to A5, SDA to A4

//--------------------Setup-------------------------
void setup() 
{
  Serial.begin(9600);
  Serial.println("RGB Colour Display Example");

  // Connect to Colour Sensor over I2C. Loop if can't.
  while (ColourSensor.begin() == false)
    {
      Serial.println("Sensor error, retrying");
      delay(1000);
    }
}


//--------------------Loop-------------------------
void loop() 
{
  // Read RGB colour measurements.
  uint16_t Red, Green, Blue, Nofilter;
  ColourSensor.getRawData(&Red,&Green,&Blue,&Nofilter);
  
 
  // Class instance for modifying properties of 
  // controls on a MegunoLink interface panel.
  InterfacePanel UI;

  // Update numerical RGB display values. 
  UI.SetNumber("RedNud", (long)Red);
  UI.SetNumber("GreenNud", (long)Green);
  UI.SetNumber("BlueNud", (long)Blue);
  

  // Gamma-encode the RGB values so the colour 
  // recreation square looks the right colour on a 
  // display. Maintain 0-255 value range.
  uint16_t RedEncoded = GammaEncode(Red, 2.2, 255);
  uint16_t GreenEncoded = GammaEncode(Green, 2.2, 255);
  uint16_t BlueEncoded = GammaEncode(Blue, 2.2, 255);
  
  // Convert binary numbers into hexidecimal strings.
  // Each must be two hex digits long.
  String RedHex = HexString(RedEncoded, 2);
  String GreenHex = HexString(GreenEncoded, 2);
  String BlueHex = HexString(BlueEncoded, 2);
  
  // Set ColourPanel Background Colour.
  // Colours can be by name or hex string.
  // Hex values need to be formatted RRGGBB.
  String RGBHex = RedHex + GreenHex + BlueHex;
  UI.SetBackColor("ColourPanel", RGBHex.c_str());
}


//--------------------Functions-------------------------
uint16_t GammaEncode(uint16_t LinearVal, float Gamma, uint16_t MaxVal)
{
   float NormalizedVal = ((float)LinearVal)/MaxVal;
   float EncodedVal = pow(NormalizedVal, 1/Gamma);
   return (uint16_t)(EncodedVal * MaxVal);
}

String HexString(uint32_t Value, uint8_t MinmumDigits)
{
  String WorkingString = String(Value, HEX);
  while (WorkingString.length() < MinmumDigits)
    {WorkingString = '0' + WorkingString;}
  return WorkingString;
}

